@@ -5,12 +5,7 @@ import hashlib
 import logging
 import re
 import socket
-import telnetlib
-import threading
-import time
-from http.server import BaseHTTPRequestHandler, HTTPServer
 from typing import (
-    TYPE_CHECKING,
     Any,
     Callable,
     Dict,
@@ -23,10 +18,9 @@ from typing import (
     Union,
 )
 
-from .results import BrokenServiceException
+import pwnlib
 
-if TYPE_CHECKING:  # pragma: no cover
-    import requests
+from .results import BrokenServiceException, EnoException
 
 PORT_MAX = 65535
 
@@ -71,6 +65,19 @@ def assert_equals(
         )
 
 
+def assert_true(
+    expression: Any, message: Optional[str] = None, autobyteify: bool = False
+) -> None:
+    """
+    Raise a BrokenServiceException if expression is not True.
+
+    :param o1: the element that should be True
+    :param message: The exception message in case of an error (optional)
+    :param autobyteify: will call ensure_bytes on both parameters.
+    """
+    assert_equals(expression, True, message, autobyteify)
+
+
 def ensure_bytes(obj: Union[bytes, str, Any]) -> bytes:
     """
     Convert an object to bytes.
@@ -89,22 +96,6 @@ def ensure_bytes(obj: Union[bytes, str, Any]) -> bytes:
     return str(obj).encode("utf-8")
 
 
-def ensure_unicode(obj: Union[bytes, str, Any]) -> str:
-    """
-    Convert an object to an utf-8-encoded string.
-
-    :param obj: str or bytes (or anything else) to convert to string representation
-    :return: the string representation of the object
-    """
-    if obj is None:
-        raise ValueError("Cannot stringify None")
-    if isinstance(obj, bytes):
-        return obj.decode("utf-8")
-    if isinstance(obj, str):
-        return obj
-    return str(obj)
-
-
 def ensure_valid_filename(s: str, min_length: int = 3) -> str:
     """
     Get a valid file name from the input.
@@ -122,14 +113,13 @@ def ensure_valid_filename(s: str, min_length: int = 3) -> str:
     return s
 
 
-def snake_caseify(camel: Union[str, bytes]) -> str:
+def snake_caseify(camel: str) -> str:
     """
     Turn camels into snake (-cases).
 
     :param camel: camelOrSnakeWhatever
     :return: camel_or_snake_whatever
     """
-    camel = ensure_unicode(camel)
     half_snake = re.sub("(.)([A-Z][a-z]+)", r"\1_\2", camel)
     return re.sub("([a-z0-9])([A-Z])", r"\1_\2", half_snake).lower()
 
@@ -181,96 +171,11 @@ def debase64ify(
         return base64.b64decode(s).decode("utf-8")
 
 
-def start_daemon(target: Callable[..., Any]) -> threading.Thread:
+class SimpleSocket(pwnlib.tubes.remote.remote):
     """
-    Start a thread as daemon.
+    Convenient socket wrapper using pwnlib's tubes.remote
 
-    :param target: the function
-    :return: the started thread
-    """
-    t = threading.Thread(target=target)
-    t.daemon = True
-    t.start()
-    return t
-
-
-def serve_once(
-    html: Union[str, bytes, "requests.Response"],
-    start_port: int = 5000,
-    autoincrement_port: bool = True,
-    content_type: str = "text/html",
-    headers: Optional[Dict[str, str]] = None,
-    logger: Optional[logging.Logger] = None,
-) -> int:
-    """
-    Render Text in the users browser.
-
-    Opens a web server that serves a HTML string once and shuts down after the first request.
-    The port will be open when this function returns. (though serving the request may take a few mils)
-
-    :param html: The html code to deliver on the initial request
-    :param start_port: The port it should try to listen on first.
-    :param autoincrement_port: If the port should be increased if the server cannot listen on the provided start_port
-    :param content_type: The content type this server should report (change it if you want json, for example)
-    :param headers: Additional headers as {header_key: value} dict.
-    :param logger: the optional logger to redirect logs to.
-    :return: The port the server started listening on
-    """
-    # see https://github.com/psf/requests/issues/2925
-    import requests
-
-    logger_: logging.Logger = logger or utilslogger
-    headers_: Dict[str, str] = headers or {}
-    if isinstance(html, requests.Response):
-        payload: bytes = html.text.encode("UTF-8")
-    elif isinstance(html, str):
-        payload = html.encode("UTF-8")
-    else:
-        payload = html
-
-    class OutputHandler(BaseHTTPRequestHandler):
-
-        # noinspection PyPep8Naming
-        def do_GET(self) -> None:
-            self.send_response(200)
-            self.send_header("Content-type", content_type)
-            for key, value in headers_.items():
-                self.send_header(key, value)
-            self.end_headers()
-            self.wfile.write(payload)
-            logger_.info("Served HTTP once. Stopping.")
-            start_daemon(self.server.shutdown)
-
-    for port in range(start_port, PORT_MAX):
-        try:
-            server = HTTPServer(("", port), OutputHandler)
-            logging.debug("Serving {} bytes on port {}".format(len(payload), port))
-            start_daemon(server.serve_forever)
-            time.sleep(0.1)  # some extra time thrown in for good measure. :)
-            return port
-        except OSError as ex:
-            if not autoincrement_port:
-                logger_.info(
-                    "Serve once was not set to automatically increment port {} but faced socket exception{}".format(
-                        start_port, ex
-                    ),
-                    exc_info=True,
-                    stack_info=True,
-                )
-                break
-
-    raise OSError(
-        "No unused port found, start_port={}, autoincrement_port={}".format(
-            start_port, autoincrement_port
-        )
-    )
-
-
-class SimpleSocket(telnetlib.Telnet):
-    """
-    Telnetlib with some additions.
-
-    Read Telnetlib doku for more.
+    Read Pwnlib documentation for more info.
     """
 
     # pylint:  disable=protected-access
@@ -281,6 +186,8 @@ class SimpleSocket(telnetlib.Telnet):
         timeout: float = socket._GLOBAL_DEFAULT_TIMEOUT,  # type: ignore
         logger: Optional[logging.Logger] = None,
         timeout_fun: Optional[Callable[[], float]] = None,
+        *args: Tuple[Any, ...],
+        **kwargs: Dict[str, Any],
     ) -> None:
         """
         Initialize a new SimpleSocket Object.
@@ -291,8 +198,8 @@ class SimpleSocket(telnetlib.Telnet):
         :param logger: The optional logger to use
         :param timeout_fun: function that will output the current timeout on each call.
         """
-        super().__init__(host, port, timeout)  # type: ignore
-        self.socket: socket.socket = super().get_socket()
+        super().__init__(host, port, timeout=timeout, *args, **kwargs)
+        self.socket: socket.socket = self.sock  # alias
         if logger:
             self.logger = logger
         else:
@@ -309,7 +216,7 @@ class SimpleSocket(telnetlib.Telnet):
         if self.timeout_fun:
             return self.timeout_fun()
         else:
-            return self.timeout  # type: ignore
+            return self.timeout
 
     def readline_expect(
         self,
@@ -374,18 +281,31 @@ class SimpleSocket(telnetlib.Telnet):
         if timeout is None:
             timeout = self.current_default_timeout
 
-        # Make sure all strings are bytes, ignore compiled Regexes.
-        regexes_: List[Union[Pattern[bytes], bytes]] = [
-            ensure_bytes(x) if isinstance(x, str) else x for x in regexes
+        if type(regexes) is str or type(regexes) is bytes:
+            raise EnoException(
+                "expect() takes a list(!) of strings or patterns as a parameter"
+            )
+
+        patterns: List[Pattern[bytes]] = [
+            v if hasattr(v, "search") else re.compile(ensure_bytes(v)) for v in regexes  # type: ignore
         ]
 
-        return super().expect(list=regexes_, timeout=timeout)  # type: ignore
+        def patterns_match(buf: bytes) -> bool:
+            for p in patterns:
+                res = p.search(buf)
+                if res is not None:
+                    print(res)
+                    return True
+            return False
+
+        return super().recvpred(patterns_match, timeout=timeout)
 
     def read_until(
         self, match: Union[bytes, str], timeout: Optional[float] = None
     ) -> bytes:
         """
-        Read until the expected string has been seen, or a timeout is hit (default is default socket timeout).
+        Read until the expected string has been seen, or a timeout is hit
+        (default is default socket timeout).
 
         :param match: what to look for.
         :param timeout: default socket timeout override
@@ -393,9 +313,11 @@ class SimpleSocket(telnetlib.Telnet):
             possibly the empty string.  Raise EOFError if the connection
             is closed and no cooked data is available.
         """
+
         if timeout is None:
             timeout = self.current_default_timeout
-        return super().read_until(ensure_bytes(match), timeout)  # type: ignore
+
+        return super().recvuntil(match, timeout=timeout)
 
     def read_n_lines(
         self, line_count: int, delimiter: Union[str, bytes] = b"\n"
@@ -415,7 +337,7 @@ class SimpleSocket(telnetlib.Telnet):
 
         :return: the complete content until EOF
         """
-        return super().read_all()
+        return super().recvall()
 
     def write(self, buffer: Union[str, bytes]) -> None:
         """
@@ -426,4 +348,4 @@ class SimpleSocket(telnetlib.Telnet):
 
         :param buffer: The buffer to write
         """
-        super().write(ensure_bytes(buffer))
+        super().send(ensure_bytes(buffer))
